@@ -29,6 +29,7 @@ class CreditScoring(Fairness):
 
         fair_threshold: int or float
                 Value between 0 and 100. If a float between 0 and 1 (not inclusive) is provided, it is converted to a percentage and the p % rule is used to calculate the fairness threshold value.        
+                If an integer between 1 and 100 is provided, it is converted to a percentage and the p % rule is used to calculate the fairness threshold value.
 
         Instance Attributes
         --------------------
@@ -151,7 +152,7 @@ class CreditScoring(Fairness):
             "fairness_metric_value_input":[(dict,), None]}
         
         self.k = Constants().k
-        self.array_size = Constants().array_size
+        self.array_size = Constants().perf_dynamics_array_size
         self.decimals = Constants().decimals  
 
         self._rejection_inference_flag = {}
@@ -170,8 +171,10 @@ class CreditScoring(Fairness):
         Wrapper function to perform all checks using dictionaries of datatypes & dictionary of values.
         This function does not return any value. Instead, it raises an error when any of the checks from the Utility class fail.
         """
+        #check datatype of input variables to ensure they are of the correct datatype
         check_datatype(self)
 
+        #check datatype of input variables to ensure they are reasonable
         check_value(self)
 
         #check for model_params
@@ -180,8 +183,10 @@ class CreditScoring(Fairness):
         if mp_given != mp_expected:
             self.err.push('length_error', var_name="model_params", given=str(mp_given), expected=str(mp_expected), function_name="_check_input")
 
+        #check for conflicting input values
         self._base_input_check()
 
+        #check if input variables will the correct fair_metric_name based on fairness tree
         self._fairness_metric_value_input_check()
 
         # check if y_pred is not None 
@@ -193,19 +198,23 @@ class CreditScoring(Fairness):
             if self.model_params[0].y_prob.dtype.kind == "i":
                 self.err.push('type_error', var_name="y_prob", given= "type int", expected="type float", function_name="_check_input")
 
+        #print any exceptions occured
         self.err.pop()
 
     def _check_special_params(self):
         """
         Perform data type and value checks for special params. 
         """
+        #check that spl_params if provided contains dictionaries, otherwise throw exception
         for i in self.spl_params.keys() :
             if type(self.spl_params[i]) != dict and  type(self.spl_params[i]) != type(None):
                 self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                 self.err.push('value_error', var_name=str(i), given=type(self.spl_params[i]), expected="dict", function_name="_check_special_params")
         
+        #print any exceptions occured
         self.err.pop()
 
+        #check that base_default_rate (under spl params) contains the protected variable under p_var, otherwise throw exception
         if self.spl_params['base_default_rate'] is not None:
             for var in self.model_params[0].p_var:
                 if self._rejection_inference_flag[var] == True:
@@ -213,17 +222,20 @@ class CreditScoring(Fairness):
                         self._rejection_inference_flag[var] = False 
                         self.err.push('value_error', var_name='base_default_rate', given="values for " + str(list(self.spl_params['base_default_rate'].keys())), expected="values for " + str( self.model_params[0].p_var), function_name="_check_special_params")
         
-        #check for length of num_applicants
+        #check that num_applicants (under spl params) contains the protected variable under p_var, otherwise throw exception        
         if self.spl_params['num_applicants'] is not None:
             for var in self.model_params[0].p_var:
                 if self._rejection_inference_flag[var] == True:
                     if var not in self.spl_params['num_applicants'].keys():
                         self._rejection_inference_flag[var] = False 
                         self.err.push('value_error', var_name='num_applicants', given="values for " + str(list(self.spl_params['num_applicants'].keys())), expected="values for " + str(self.model_params[0].p_var), function_name="_check_special_params")
+        #print any exceptions occured
         self.err.pop()
 
+        #specify datatypes that are accepted for base_default_rate and num_applicants
         num_applicants_type = (int, float,)
         base_default_rate_type = (float,)
+        #check that num_applicants contains correct datatype, otherwise throw exception
         if self.spl_params['num_applicants'] is not None:
             for i in self.spl_params["num_applicants"]:
                 if self._rejection_inference_flag[i] == True :
@@ -237,6 +249,7 @@ class CreditScoring(Fairness):
                                         given=str(type(self.spl_params["num_applicants"].get(i)[j])),
                                         expected=str(num_applicants_type), function_name="_check_special_params")
             
+        #check that base_default_rate contains correct datatype, otherwise throw exception
         if self.spl_params['base_default_rate'] is not None:
             for i in self.spl_params["base_default_rate"]:
                 if self._rejection_inference_flag[i] == True :
@@ -247,8 +260,10 @@ class CreditScoring(Fairness):
                                         given=str(type(self.spl_params["base_default_rate"].get(i)[j])),
                                         expected=str(base_default_rate_type), function_name="_check_special_params")
 
+        #specify range of values that are accepted for base_default_rate and num_applicants
         num_applicants_range = (0, np.inf)
         base_default_rate_range = (0, 1)
+        #check that num_applicants values are within range, otherwise throw exception
         if self.spl_params["num_applicants"] is not None:
             for k, l in self.spl_params['num_applicants'].items() :
                 if self._rejection_inference_flag[k] == True :
@@ -258,6 +273,7 @@ class CreditScoring(Fairness):
                             self.err.push('value_error', var_name='num_applicants for ' + str(k), given=str(m),
                                           expected=str(num_applicants_range), function_name="_check_special_params")
 
+        #check that base_default_rate values are within range, otherwise throw exception
         if self.spl_params["base_default_rate"] is not None:
             for k, l in self.spl_params['base_default_rate'].items() :
                 if self._rejection_inference_flag[k] == True :
@@ -281,8 +297,8 @@ class CreditScoring(Fairness):
                     if len(self.spl_params['num_applicants'][var]) != 2:
                         self._rejection_inference_flag[var] = False 
                         self.err.push('length_error', var_name='num_applicants', given=len(self.spl_params['num_applicants'][var]), expected='2', function_name="_check_special_params")
-        #check for num of applicants
-
+        
+        #check for num of applicants if values in each index are consistent
         val_lst = []
         if self.spl_params['num_applicants'] is not None:
             for key, val in self.spl_params['num_applicants'].items():
@@ -300,6 +316,7 @@ class CreditScoring(Fairness):
             except TypeError:
                 pass
 
+        #check for common base default rate based in spl params input values. If inconsistent, throw exception
         rejection_inference_filter = {k: v for k, v in self._rejection_inference_flag.items() if v == True}
         if len(rejection_inference_filter) > 0 :
             #Check for common base default rate
@@ -320,6 +337,7 @@ class CreditScoring(Fairness):
                         self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                         self.err.push('conflict_error', var_name_a="Common base default rates", some_string="inconsistent values", value=[round(br_value[1], 5), round(check_cbdr[val], 5)], function_name="_check_special_params")
 
+        #check if num_applicants is more than length of y_true, otherwise throw exception
         na_var = self.spl_params['num_applicants']
         exp_out = np.array(self.model_params[0].y_true)
         if na_var is not None:
@@ -341,9 +359,10 @@ class CreditScoring(Fairness):
                         self.err.push('value_error_compare', var_name_a="Total number of applicants",
                                         var_name_b="total number of approvals", function_name="_check_special_params")    
         
+        #check if spl params provided are realistic, otherwise throw exception
         if self.model_params[0].y_pred is not None:
             tn, fp, fn, tp = self._get_confusion_matrix(self.model_params[0].y_true, self.model_params[0].y_pred, self.model_params[0].sample_weight)
-                # check for acceptance cohort rate
+            #check for acceptance cohort rate
             if fn < 0 or tn < 0:
                 self._rejection_inference_flag = self._rejection_inference_flag.fromkeys(self._rejection_inference_flag, False)
                 self.err.push('conflict_error', var_name_a="base_default_rate and/or num_applicants", some_string="unrealistic_input",
@@ -363,6 +382,7 @@ class CreditScoring(Fairness):
                         if group_default_rate_u > self.spl_params['base_default_rate'].get(curr_p_var)[1]:
                             self.err.push('conflict_error', var_name_a="base_default_rate", some_string="unrealistic_input",
                                             value='', function_name="_check_special_params")
+        #print any exceptions occured
         self.err.pop()
 
     def _select_fairness_metric_name(self):
@@ -400,7 +420,7 @@ class CreditScoring(Fairness):
         -------
         Confusion matrix metrics based on privileged and unprivileged groups or a list of None if curr_p_var == None
         """
-        
+        #confusion matrix will only run for classification models        
         if self._model_type_to_metric_lookup[self.model_params[0].model_type][0] == "classification" :
 
             if 'y_true' in kwargs:
