@@ -15,7 +15,6 @@ class PerformanceMetrics:
     map_perf_metric_to_group : dictionary
             Maps the performance metric names to their corresponding full names, metric group eg classification or regression metric types, and whether it can be a primary metric
     """
-
     map_perf_metric_to_group = {'selection_rate':('Selection Rate', 'classification', True),
                                 'accuracy': ('Accuracy', 'classification', True),
                                 'balanced_acc': ('Balanced Accuracy', 'classification', True),
@@ -85,7 +84,6 @@ class PerformanceMetrics:
         label_size : int, default=None
                 Number of labels allowed
         """
-
         self.result = None
 
         self.map_perf_metric_to_method = {'selection_rate': self._compute_selection_rate,
@@ -124,7 +122,6 @@ class PerformanceMetrics:
         self.revenue = None
         self.label_size = None
         self.use_case_object = use_case_object
-        # use_case_object
 
     def execute_all_perf(self, n_threads, seed, eval_pbar):
         """
@@ -156,15 +153,18 @@ class PerformanceMetrics:
         self._use_case_metrics = self.use_case_object._use_case_metrics
         self.y_train = [model.y_train for model in self.use_case_object.model_params]
 
+        #initialize result structure
         self.result = {}
         self.result["perf_metric_values"] = {}
         for j in self._use_case_metrics['perf']:
                 if j in self.map_perf_metric_to_method.keys():
                     self.result["perf_metric_values"][j] = []
+        #update progress bar by 10
         eval_pbar.update(10)
 
         n_threads = check_multiprocessing(n_threads)
         n = len(self.use_case_object.model_params[0].y_true)
+        #split k into k-1 times of random indexing compute and 1 time of original array compute
         if n_threads >= 1 and self.use_case_object.k > 1:
             indices = []
             np.random.seed(seed)
@@ -186,24 +186,30 @@ class PerformanceMetrics:
                         metric_obj = self
                     else:
                         metric_obj = deepcopy(self)
+                    #submit each thread's work to thread pool
                     if len(indexes[k]) > 0:
                         threads.append(executor.submit(PerformanceMetrics._execute_all_perf_map, metric_obj=metric_obj, index=indexes[k], eval_pbar=eval_pbar, worker_progress=worker_progress))
 
+                #retrive results from each thread
                 for thread in threads:
                     for key, v in thread.result().items():
                         self.result['perf_metric_values'][key] = self.result['perf_metric_values'][key] + v
         else:
+            #if multithreading is not triggered, directly update the progress bar by 24
             eval_pbar.update(24)
 
+        #run 1 time of original array to compute performance metrics
         PerformanceMetrics._execute_all_perf_map(self, [np.arange(n)], eval_pbar, 1)
+
+        #generate the final performace metrics values and their CI based on k times of computation
         for j in self.result['perf_metric_values'].keys():
             if self.result['perf_metric_values'][j][-1] is None :
                 self.result['perf_metric_values'][j] = (None, None)
             else :
-               self.result['perf_metric_values'][j] = (self.result['perf_metric_values'][j][-1],  2*np.std(self.result['perf_metric_values'][j]))            
+               self.result['perf_metric_values'][j] = (self.result['perf_metric_values'][j][-1],  2*np.std(self.result['perf_metric_values'][j]))
         self.label_size = self.use_case_object._model_type_to_metric_lookup[self.use_case_object.model_params[0].model_type][1]
         self.result["class_distribution"] = self._get_class_distribution(self.y_true[-1], self.use_case_object.model_params[-1].pos_label2)
-        self.result['weighted_confusion_matrix'] = { "tp":self.tp, "fp": self.fp, "tn": self.tn, "fn": self.fn }                
+        self.result['weighted_confusion_matrix'] = { "tp":self.tp, "fp": self.fp, "tn": self.tn, "fn": self.fn }
         self.result["calibration_curve"] = self._calibration_func(self.y_true[0], self.y_prob[0], n_bins=10)
         self.result["perf_dynamic"] = self._performance_dynamics()
         eval_pbar.update(6)
@@ -213,17 +219,17 @@ class PerformanceMetrics:
         Maps each thread's work for execute_all_perf()
         Parameters
         ----------
-        metric_obj :
-        
-        index : 
-
+        metric_obj : PerformanceMetrics object
+        index : array of shape (n,m)
         eval_pbar : tqdm object
                 Progress bar
-
-        worker_progress :
+        worker_progress : int
+                Progress bar progress for each thread
         """
+        #get each iteration's progress in 2 decimals to update the progress bar
         prog = round(worker_progress/(len(index)),2)
         for idx in index:
+            #prepare data
             metric_obj.y_true = [model.y_true[idx] for model in metric_obj.use_case_object.model_params]
             metric_obj.y_prob = [model.y_prob[idx] if model.y_prob is not None else None for model in metric_obj.use_case_object.model_params] 
             metric_obj.y_pred = [model.y_pred[idx] if model.y_pred is not None else None for model in metric_obj.use_case_object.model_params]
@@ -231,6 +237,7 @@ class PerformanceMetrics:
             metric_obj.e_lift = metric_obj.use_case_object.e_lift[idx] if metric_obj.use_case_object.e_lift is not None else None
             metric_obj.pred_outcome = {k: v[idx] for k, v in metric_obj.use_case_object.pred_outcome.items()} if metric_obj.use_case_object.pred_outcome is not None else {None}
 
+            #compute performace metrics
             metric_obj.tp, metric_obj.fp, metric_obj.tn, metric_obj.fn = metric_obj.use_case_object._get_confusion_matrix(metric_obj.y_true[0], metric_obj.y_pred[0], metric_obj.sample_weight[0])            
             for j in metric_obj._use_case_metrics['perf']:
                 if j in metric_obj.map_perf_metric_to_method.keys():
@@ -256,7 +263,6 @@ class PerformanceMetrics:
         ----------
         perf_metric_values : dictionary of tuples
             Stores both the performance metric value and the corresponding confidence interval for every metric in include_metrics
-
         """
         return self.map_perf_metric_to_method[metric_name](**kwargs)
     
@@ -273,9 +279,7 @@ class PerformanceMetrics:
         ----------
         _compute_accuracy : float
                 The performance metric value
-
         """
-        
         if 'y_pred_new' in kwargs:
             tp, fp, tn, fn = self.use_case_object._get_confusion_matrix(y_true=self.y_true[0], y_pred=kwargs['y_pred_new'][0], sample_weight = self.sample_weight[0])
         else:
@@ -353,7 +357,6 @@ class PerformanceMetrics:
         f1_scr = 2 * ((tp / (tp + fp)) * (tp / (tp + fn))) / ((tp / (tp + fp)) + tp / (tp + fn))
         return f1_scr
 
-
     def _compute_precision(self, **kwargs):
         """
         Computes the precision
@@ -376,7 +379,6 @@ class PerformanceMetrics:
         precision = tp / (tp + fp)
         return precision
 
-
     def _compute_recall(self, **kwargs):
         """
         Computes the recall
@@ -395,12 +397,10 @@ class PerformanceMetrics:
             tp, fp, tn, fn = self.use_case_object._get_confusion_matrix(y_true=self.y_true[0], y_pred=kwargs['y_pred_new'][0], sample_weight = self.sample_weight[0])
         else:
             tp, fp, tn, fn = self.tp, self.fp, self.tn, self.fn
-        
 
         recall = tp / (tp + fn)
         return recall
 
-
     def _compute_tnr(self, **kwargs):
         """
         Computes the true negative rate or specificity
@@ -420,11 +420,9 @@ class PerformanceMetrics:
         else:
             tp, fp, tn, fn = self.tp, self.fp, self.tn, self.fn
             
-
         tnr = tn / (tn + fp)
         return tnr
 
-
     def _compute_fnr(self, **kwargs):
         """
         Computes the false negative rate or miss-rate
@@ -465,7 +463,6 @@ class PerformanceMetrics:
         _compute_emp_lift : float
                 The performance metric value
         """
-            
         y_true = self.y_true[1]
         e_lift = self.e_lift
         
@@ -490,7 +487,6 @@ class PerformanceMetrics:
 
         return emp_lift
 
-
     def _compute_expected_profit(self, selection_threshold = None, **kwargs):
         """
         Computes expected profit from the revenue and cost values
@@ -534,7 +530,6 @@ class PerformanceMetrics:
         profit = sum(profit_RcT - profit_RcC)
 
         return profit
-        
 
     def _compute_expected_selection_rate(self, selection_threshold = None, **kwargs):
         """
@@ -554,7 +549,6 @@ class PerformanceMetrics:
         _compute_expected_selection_rate : float
             The performance metric value
         """
-
         e_lift = self.e_lift
         
         if 'y_pred_new' in kwargs:
@@ -571,7 +565,6 @@ class PerformanceMetrics:
         bools_avg = sum(bools)/len(bools)
         return bools_avg
 
-
     def _compute_negative_predictive_value(self, **kwargs):
         """
         Computes the negative predictive value
@@ -694,9 +687,8 @@ class PerformanceMetrics:
         else: 
             sample_weight = self.sample_weight[0]
             roc_auc = roc_auc_score(y_true=y_true, y_score=y_prob, sample_weight=self.sample_weight[0])
-        
-        return roc_auc
 
+        return roc_auc
 
     def _compute_log_loss(self, **kwargs):
         """
@@ -728,8 +720,7 @@ class PerformanceMetrics:
             log_loss_score = log_loss(y_true=y_true, y_pred=y_prob, sample_weight=self.sample_weight[0])
 
         return log_loss_score
-          
-        
+
     def _performance_dynamics(self):
         """
         Computes the dynamic performance metrics based on different threshold values
@@ -744,12 +735,12 @@ class PerformanceMetrics:
             return None
         else:
             d = {}
-            d['perf_metric_name'] = []  
+            d['perf_metric_name'] = []
             d['threshold'] = []
             d['perf'] = []
             d['selection_rate'] = []
             if metric_group == 'classification':
-                threshold = np.linspace(0.3, 0.7, Constants().array_size)
+                threshold = np.linspace(Constants().classify_min_threshold, Constants().classify_max_threshold, Constants().perf_dynamics_array_size)
                 d['threshold'] = threshold
                 for i in range(len(threshold)):
                     y_pred_new = [j > threshold[i] for j in self.y_prob[0]]  
@@ -763,16 +754,15 @@ class PerformanceMetrics:
                 if self.y_prob[1] is None:
                     return None
                 else:
+                    threshold = np.linspace(Constants().uplift_min_threshold, Constants().uplift_max_threshold, Constants().perf_dynamics_array_size)
                     if self.perf_metric_name == 'emp_lift':
                         d['perf_metric_name'] = 'emp_lift'
-                        threshold = np.linspace(-0.3, 0.3, Constants().array_size)
                         d['threshold'] = threshold
                         for i in range(len(threshold)):
                             d['selection_rate'] += [self._compute_expected_selection_rate(threshold[i])]
                             d['perf'] += [self._compute_emp_lift(threshold[i])]
                     else:   
                         d['perf_metric_name'] = 'expected_profit'
-                        threshold = np.linspace(-0.3, 0.3, Constants().array_size)
                         d['threshold'] = threshold
                         for i in range(len(threshold)):
                             d['selection_rate'] += [self._compute_expected_selection_rate(threshold[i])]
@@ -815,7 +805,6 @@ class PerformanceMetrics:
 
         return calibration_curve_bin
 
-
     def _get_class_distribution(self, y_true, pos_label = 1):
         """
         Calculates the proportion of favourable and unfavourable labels in y_true.
@@ -840,4 +829,4 @@ class PerformanceMetrics:
             y_true_counts = pd.Series(y_true).value_counts(normalize = True)
             y_true_counts = y_true_counts.reset_index().replace({1: 'pos_label', 0:'neg_label'}).set_index('index')
             return y_true_counts[0].to_dict()
-        
+
